#include <Wire.h>
#include "UPing.h"
#include "LCD_I2C.h"

LCD_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);
UPing sonar(3, 2, 400); // 트리거 핀은 3번, 에코 핀은 2번, 최대측정거리는 4미터 입니다.

const int motorLeftF =6; // 왼쪽 모터의 전진 신호는 6번으로 정의
const int motorLeftB =9; // 왼쪽 모터의 후진 신호는 9번으로 정의
const int motorRightF=11; // 오른쪽 모터의 전진 신호는 11번으로 정의
const int motorRightB=10; // 오른쪽 모터의 전진 신호는 10번으로 정의

const long turn_time = 500; //오른쪽으로 방향찾기 위해 회전하는 시간. 90도 정도 돌때까지 걸리는 시간을 대략으로 측정한다.
const long left_turn_time = 500; //왼쪽으로 방향찾기 위해 회전하는 시간. 90도 정도 돌때까지 걸리는 시간을 대략으로 측정한다.
const long turning_distance = 320; //ping타임은 장애물 까지 거리(cm) * 56 마이크로 초입니다. 280이면 약 5cm
const long backward_distance = 160;  //너무 가까우면 뒤로 가기


// 바퀴가 움직이는 속도를 정해줍니다.
// 0~255까지 analogWrite에 사용하는 값. 100이상이 되어야 움직입니다. 
//속도가 적당하지 않거나 똑바로 가지 않으면 값을 조절합니다.
const int rigth_speed = 200; 
const int left_speed = 200; 

void setup() {
	randomSeed(analogRead(1));
	lcd.backlight();
	lcd.begin(16, 2);
	lcd.setCursor(0, 0); //글자의 시작위치는 0번째줄 0번째 문자입니다.
	lcd.print("I'm Yellow Cat  ");
	lcd.setCursor(0, 1); //글자의 시작위치는 0번째줄 0번째 문자입니다.
	lcd.print("Start Ultrasonic");

	pinMode(motorRightF, OUTPUT);
	pinMode(motorRightB, OUTPUT);
	pinMode(motorLeftF, OUTPUT);
	pinMode(motorLeftB, OUTPUT);
	left(0); right(0); //정지 
	delay(1000);
	lcd.setCursor(0, 0); //글자의 시작위치는 0번째줄 0번째 문자입니다.
	lcd.print("OK              ");
	lcd.setCursor(0, 1); //글자의 시작위치는 0번째줄 0번째 문자입니다.
	lcd.print("Go!             ");

}

void loop() {
	long ping_distance,  max_distance  ;
	unsigned long when_search_started, ellapsed,ellapsed_till_max_distance, time_to_go_back;
	char direction;
	bool found_max = false; 
	bool is_turning_right;
	ping_distance = sonar.ping(); //ping_median(2) : 두번 초음파로 측정한 평균거리, ping_distance/56  /cm 


	if ( ping_distance < turning_distance) {
		if ( ping_distance < backward_distance) { //너무 가까우면 뒤로 약간 갑니다.
			lcd.setCursor(0, 0); //글자의 시작위치는 0번째줄 0번째 문자입니다.
			lcd.print("I am going back!");
			lcd.setCursor(0, 1); 
			lcd.print("                "); //공백으로 지운다.
			lcd.setCursor(0, 1); //다시 두번째줄 처음으로 돌아간다.
			lcd.print(ping_distance/56);lcd.print("cm  ");
			left(-left_speed); right(-rigth_speed);  //양쪽 바퀴를 뒤로 후진시킵니다.
		}else{
			lcd.setCursor(0, 0); //글자의 시작위치는 0번째줄 0번째 문자입니다.
			lcd.print("Oops! Blocked!  "); //LCD에 글자를 표시합니다.
			lcd.setCursor(0, 1); 
			lcd.print("                "); //공백으로 지운다.
			lcd.setCursor(0, 1); //다시 두번째줄 처음으로 돌아간다.
			lcd.print(ping_distance/56);lcd.print("cm  ");
			is_turning_right=random(2); //동전 던지기처럼  0 or 1 이 무작위로 결정됩니다.
			if(is_turning_right){ //만약 is_turning_right 가 1(참)이라면
				left(left_speed); right(-rigth_speed); //오른쪽으로 제자리에서 회전		
			}else{
				left(-left_speed); right(rigth_speed); //왼쪽으로 제자리에서 회전		
			}		
		}
		delay(turn_time); //회전할 시간 만큼 유지
		delay(30);
	}else{
		left(left_speed); right(rigth_speed); //전진
	}
	
	delay(20); 
}

void left(int speed){ //왼쪽 모터 제어 
	if (speed >= 0) { //전진
		digitalWrite(motorLeftB, LOW); //왼쪽 후진 신호 OFF
		analogWrite(motorLeftF, speed); //전진 신호 speed의 세기로 회전
	} else { //후진인 경우
		digitalWrite(motorLeftF, LOW); //왼쪽 후진 신호 OFF
		analogWrite(motorLeftB, -speed); //후진 신호 양수로 변경된 speed의 세기로 회전
	}
}
void right(int speed){ //오른쪽 모터 제어 
	if (speed >= 0) { //전진
		digitalWrite(motorRightB, LOW); //오른쪽 후진 신호 OFF
		analogWrite(motorRightF, speed); //전진 신호 speed의 세기로 회전 
	} else { //후진인 경우
		digitalWrite(motorRightF, LOW); //오른쪽 전진 신호 OFF
		analogWrite(motorRightB, -speed); //후진 신호 양수로 변경된 speed의 세기로 회전
	}
}
